package com.aliyun.iotx.linkvisual.ipc;

import android.Manifest;
import android.app.Activity;
import android.content.pm.PackageManager;

import android.media.AudioFormat;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.support.v4.app.ActivityCompat;
import android.text.TextUtils;
import android.util.Base64;
import android.view.View;
import android.widget.Button;
import android.widget.ScrollView;
import android.widget.TextView;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.alink.dm.api.DeviceInfo;
import com.aliyun.alink.dm.api.IoTApiClientConfig;
import com.aliyun.alink.linkkit.api.ILinkKitConnectListener;
import com.aliyun.alink.linkkit.api.IoTDMConfig;
import com.aliyun.alink.linkkit.api.IoTMqttClientConfig;
import com.aliyun.alink.linkkit.api.LinkKit;
import com.aliyun.alink.linkkit.api.LinkKitInitParams;
import com.aliyun.alink.linksdk.cmp.connect.channel.MqttPublishRequest;
import com.aliyun.alink.linksdk.cmp.core.base.AMessage;
import com.aliyun.alink.linksdk.cmp.core.base.ARequest;
import com.aliyun.alink.linksdk.cmp.core.base.AResponse;
import com.aliyun.alink.linksdk.cmp.core.base.ConnectState;
import com.aliyun.alink.linksdk.cmp.core.listener.IConnectNotifyListener;
import com.aliyun.alink.linksdk.cmp.core.listener.IConnectSendListener;
import com.aliyun.alink.linksdk.tmp.api.OutputParams;
import com.aliyun.alink.linksdk.tmp.device.payload.ValueWrapper;
import com.aliyun.alink.linksdk.tmp.devicemodel.Event;
import com.aliyun.alink.linksdk.tmp.devicemodel.Property;
import com.aliyun.alink.linksdk.tmp.devicemodel.Service;
import com.aliyun.alink.linksdk.tmp.listener.IPublishResourceListener;
import com.aliyun.alink.linksdk.tmp.listener.ITResRequestHandler;
import com.aliyun.alink.linksdk.tmp.listener.ITResResponseCallback;
import com.aliyun.alink.linksdk.tmp.utils.ErrorInfo;
import com.aliyun.alink.linksdk.tools.AError;
import com.aliyun.alink.linksdk.tools.ALog;
import com.aliyun.iotx.linkvisual.ipc.utils.EnvUtils;
import com.aliyun.iotx.linkvisual.ipc.utils.FileUtil;
import com.aliyun.iotx.linkvisual.ipc.utils.MockUtils;
import com.aliyun.iotx.linkvisual.ipc.utils.UploadPicUtil;
import com.aliyun.iotx.linkvisualipc.IPCDev;
import com.aliyun.iotx.linkvisualipc.audio.AudioParams;
import com.aliyun.iotx.linkvisualipc.audio.IPCLiveIntercomV2;
import com.aliyun.iotx.linkvisualipc.audio.LiveIntercomError;
import com.aliyun.iotx.linkvisualipc.audio.listener.OnLiveIntercomErrorListener;
import com.aliyun.iotx.linkvisualipc.audio.listener.OnLiveIntercomListener;
import com.aliyun.iotx.linkvisualipc.interfaces.ReportAlarmEventListener;
import com.aliyun.iotx.linkvisualipc.pic.bean.UploadInfo;
import com.aliyun.iotx.linkvisualipc.pic.interfaces.IUploadPicListener;
import com.aliyun.iotx.linkvisualipc.pic.interfaces.IUploadPicProcessCallback;
import com.aliyun.iotx.linkvisualipc.video.AudioStreamParams;
import com.aliyun.iotx.linkvisualipc.video.NoSuchStreamException;
import com.aliyun.iotx.linkvisualipc.video.OnLiveStreamListener;
import com.aliyun.iotx.linkvisualipc.video.OnStreamErrorListener;
import com.aliyun.iotx.linkvisualipc.video.OnVodStreamListener;
import com.aliyun.iotx.linkvisualipc.video.StreamError;
import com.aliyun.iotx.linkvisualipc.video.VideoStreamParams;

import java.nio.ByteBuffer;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

public class MainActivity extends Activity
        implements OnStreamErrorListener, OnLiveStreamListener, OnVodStreamListener {
    private final static String TAG = "MainActivity";
    private final static String CONNECT_ID = "LINK_PERSISTENT";

    public static String productKey = EnvUtils.getProductKey();
    public static String deviceName = EnvUtils.getDeviceName();
    public static String deviceSecret = EnvUtils.getDeviceSecret();

    public static String productSecret = null;

    private List<Property> propertyList = new LinkedList<>();
    private List<Service> serviceList = new LinkedList<>();
    private List<Event> eventList = new LinkedList<>();

    private TextView logTv;
    private Button reportEventBtn;
    private ScrollView logScroll;
    public Handler uiHandler;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        uiHandler = new Handler(getMainLooper());
        ALog.setLevel(ALog.LEVEL_DEBUG);
        initView();
        init(productKey, deviceName, deviceSecret, productSecret);
        appendLog("ProductKey:" + productKey);
        appendLog("DeviceName:" + deviceName);
        String[] permissions = {Manifest.permission.RECORD_AUDIO, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.MODIFY_AUDIO_SETTINGS};
        verifyPermissions(permissions, this);

        FileUtil.copyFile(this, "1.jpeg");
        FileUtil.copyFile(this, "2.jpeg");
        if(TextUtils.isEmpty(productKey) || TextUtils.isEmpty(deviceName) || TextUtils.isEmpty(deviceSecret)){
            appendLog(EnvUtils.help());
        }
    }

    private void initView() {
        logTv = findViewById(R.id.log_tv);
        logScroll = findViewById(R.id.log_scroll);
        reportEventBtn = findViewById(R.id.report_event_btn);
        reportEventBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {

                //上报报警事件分成两步：1.上传报警事件，2.上传图片
                //上报报警事件需要用户生成一个报警图片ID，这个报警图片ID要求是设备唯一的图片ID，建议用Unix时间戳。
                //云端收到上报事件后，会通过IUploadPicListener（需要提前注册监听）的onUpload回调通知设备端将报警图片上传到回调中url指定地址
                //注意：事件上报最小间隔为10S，低于该间隔的事件将会被忽略
                final String alarmPicId = String.valueOf(System.currentTimeMillis() / 1000);
                IPCDev.getInstance().reportAlarmEvent(alarmPicId, 1, new ReportAlarmEventListener() {
                    @Override
                    public void onSucceed() {
                        ALog.d(TAG, " report onSucceed  alarmPicId:" + alarmPicId);
                        appendLog("上报事件成功");
                    }

                    @Override
                    public void onFailed(String msg) {
                        ALog.e(TAG, " report onFailed  e:" + msg);
                        appendLog("上报事件失败  e:" + msg);
                    }
                });
            }
        });

    }

    private void init(final String productKey, final String deviceName, final String deviceSecret,
                      String productSecret) {

        // 构造三元组信息对象
        DeviceInfo deviceInfo = new DeviceInfo();
        // 产品类型
        deviceInfo.productKey = productKey;
        // 设备名称
        deviceInfo.deviceName = deviceName;
        // 设备密钥
        deviceInfo.deviceSecret = deviceSecret;
        // 产品密钥
        deviceInfo.productSecret = productSecret;
        //  全局默认域名
        IoTApiClientConfig userData = new IoTApiClientConfig();
        // 设备的一些初始化属性，可以根据云端的注册的属性来设置。
        /**
         * 物模型初始化的初始值
         * 如果这里什么属性都不填，物模型就没有当前设备相关属性的初始值。
         * 用户调用物模型上报接口之后，物模型会有相关数据缓存。
         * 用户根据时间情况设置
         */
        Map<String, ValueWrapper> propertyValues = new HashMap<>();
        // 示例
        //propertyValues.put("StreamVideoQuality", new ValueWrapper.EnumValueWrapper(2));
        //propertyValues.put("SubStreamVideoQuality", new ValueWrapper.EnumValueWrapper(1));
        //propertyValues.put("AlarmSwitch", new ValueWrapper.BooleanValueWrapper(1));
        //propertyValues.put("MotionDetectSensitivity", new ValueWrapper.EnumValueWrapper(5));
        //propertyValues.put("VoiceDetectionSensitivity", new ValueWrapper.EnumValueWrapper(5));
        //propertyValues.put("AlarmFrequencyLevel", new ValueWrapper.EnumValueWrapper(2));
        //propertyValues.put("MicSwitch", new ValueWrapper.BooleanValueWrapper(1));
        //propertyValues.put("SpeakerSwitch", new ValueWrapper.BooleanValueWrapper(1));
        //propertyValues.put("StatusLightSwitch", new ValueWrapper.BooleanValueWrapper(1));
        //propertyValues.put("ImageFlipState", new ValueWrapper.EnumValueWrapper(1));
        //propertyValues.put("StorageRecordMode", new ValueWrapper.EnumValueWrapper(2));
        //propertyValues.put("StorageStatus", new ValueWrapper.EnumValueWrapper(1));
        //propertyValues.put("StorageTotalCapacity", new ValueWrapper.DoubleValueWrapper(120.0));
        //propertyValues.put("StorageRemainCapacity", new ValueWrapper.DoubleValueWrapper(1.0));
        //propertyValues.put("DayNightMode", new ValueWrapper.EnumValueWrapper(2));
        //propertyValues.put("EncryptSwitch", new ValueWrapper.BooleanValueWrapper(1));

        LinkKitInitParams params = new LinkKitInitParams();
        params.deviceInfo = deviceInfo;
        params.propertyValues = propertyValues;
        params.connectConfig = userData;

        /**
         * 慎用：如不清楚是否要设置，可以不设置该参数
         * Mqtt 相关参数设置
         * 域名、产品密钥、认证安全模式等；
         */
        IoTMqttClientConfig clientConfig = new IoTMqttClientConfig(productKey, deviceName, deviceSecret);
        params.mqttClientConfig = clientConfig;
            /**
             * 设备是否支持被飞燕平台APP发现
             * 需要确保开发的APP具备发现该类型设备的权限
             */
        IoTDMConfig ioTDMConfig = new IoTDMConfig();
        // 启用设备支持发现绑定的功能
        ioTDMConfig.enableNotify = true;
        ioTDMConfig.enableThingModel = true;
        ioTDMConfig.enableLocalCommunication = false;
        params.ioTDMConfig = ioTDMConfig;

        /**
         * 设备初始化建联
         * onError 初始化建联失败，如果因网络问题导致初始化失败，需要用户重试初始化
         * onInitDone 初始化成功
         */
        LinkKit.getInstance().init(this, params, new ILinkKitConnectListener() {
            @Override
            public void onError(AError error) {
                ALog.d(TAG,
                        "onError() called with: error = [" + (error == null ? "null" : (error.getCode() + error.getMsg()))
                                + "]");
            }

            @Override
            public void onInitDone(Object data) {
                setReportEventValid(true);
                ALog.d(TAG, "onInitDone() called with: data = [" + JSON.toJSONString(data) + "]");
                getServiceList();
                setServiceHandler();

                // 设置使用对讲第二版本
                IPCDev.getInstance().setLiveIntercomVersionBeforeInit(IPCDev.LiveIntercomVersion.VERSION_2);

                // 设置双向对讲模式
                IPCDev.getInstance().setLiveIntercomModeBeforeInit(IPCLiveIntercomV2.LiveIntercomMode.DoubleTalk);

                //初始化IPC SDK
                IPCDev.getInstance().init(MainActivity.this, productKey, deviceName, deviceSecret);

                //注册上传图片监听，当App端触发拍照或者设备上报了事件以后，会在注册的回调中收到上传的回调。
                IPCDev.getInstance().registerUploadPicListener(uploadPicListener);


                /* Stream related below */

                // 设置推流错误回调
                IPCDev.getInstance().getIpcStreamManager().setOnStreamErrorListener(MainActivity.this);
                // 设置直播流事件监听
                IPCDev.getInstance().getIpcStreamManager().setOnLiveStreamListener(MainActivity.this);
                // 设置点播流事件监听
                IPCDev.getInstance().getIpcStreamManager().setOnVodStreamListener(MainActivity.this);
                // 一次性加载测试用h264视频文件
                h264HighQualityData = MockUtils.readFileAsByteArrays(getApplicationContext(), H264_720P_FILE_NAME, H264_720P_FILE_LENGTH);
                h264LowQualityData = MockUtils.readFileAsByteArrays(getApplicationContext(), H264_480P_FILE_NAME, H264_480P_FILE_LENGTH);
                // 一次性加载测试用g711a音频文件
                g711aData = MockUtils.readFileAsByteArrays(getApplicationContext(), G711A_FILE_NAME, G711A_FILE_LENGTH);
                // 一次性加载测试用AAC音频文件
                aacData = MockUtils.readFileAsByteArrays(getApplicationContext(), AAC_FILE_NAME, AAC_FILE_LENGTH);


                /* Live Intercom related below */

                // 设置语音对讲事件监听
                IPCDev.getInstance().getIpcLiveIntercom().setOnLiveIntercomListener(onLiveIntercomListener);
                // 设置语音对讲错误回调
                IPCDev.getInstance().getIpcLiveIntercom().setOnLiveIntercomErrorListener(onLiveIntercomErrorListener);

                appendLog("Linkkit初始化完成");
            }
        });
    }

    /**
     * 云端通知设备端上传图片，上报结果需要通过callback回传给SDK内部消化
     * type为上传类型，0为云端触发主动抓图，1为事件上报对应的图片上报
     */
    private IUploadPicListener uploadPicListener = new IUploadPicListener() {
        @Override
        public void onUpload(final UploadInfo uploadInfo, final IUploadPicProcessCallback callback) {
            final String url = uploadInfo.getUploadUrl();
            // 仅当type为1时有picId，与事件上报时的picId相同，用于关联
            final String picId = uploadInfo.getPicId();
            final int type = uploadInfo.getType();

            ALog.d(TAG, "url:" + url + "    picId:" + picId + "   type:" + type);
            switch (type) {
                // 由云端主动触发的抓图，无须上报事件
                case 0:
                    uiHandler.postDelayed(new Runnable() {
                        @Override
                        public void run() {
                            try {
                                UploadPicUtil.uploadImage(url, Environment.getExternalStorageDirectory()
                                                .getAbsolutePath() + "/2.jpeg", "image/jpeg",
                                        new UploadPicUtil.UploadResultCallback() {

                                            @Override
                                            public void onSucceed() {
                                                ALog.d(TAG, "uploadImage onSucceed");
                                                appendLog("抓图上传成功");
                                                if (callback != null) {
                                                    callback.onSucceed();
                                                }
                                            }

                                            @Override
                                            public void onFailed(String error) {
                                                ALog.e(TAG, "uploadImage onFailed:" + error);
                                                appendLog("抓图上传失败，error：" + error);
                                                if (callback != null) {
                                                    callback.onFailed(error);
                                                }
                                            }
                                        });
                            } catch (Exception e) {
                                if (callback != null) {
                                    callback.onFailed(e.toString());
                                }
                                ALog.e(TAG, "uploadImage onFailed:" + e.toString());
                                appendLog("抓图上传失败，error：" + e.toString());
                            }

                        }
                    }, 1000);

                    break;
                //由事件上报触发的图片上报
                case 1:
                    try {
                        UploadPicUtil.uploadImage(url, Environment.getExternalStorageDirectory()
                                .getAbsolutePath() + "/1.jpeg", "image/jpeg", new UploadPicUtil.UploadResultCallback() {
                            @Override
                            public void onSucceed() {
                                ALog.d(TAG, "uploadImage onSucceed");
                                appendLog("事件图片上报成功");
                                if (callback != null) {
                                    callback.onSucceed();
                                }
                            }

                            @Override
                            public void onFailed(String error) {
                                ALog.e(TAG, "uploadImage onFailed:" + error);
                                appendLog("事件图片上报失败，error：" + error);
                                if (callback != null) {
                                    callback.onFailed(error);
                                }
                            }
                        });
                    } catch (Exception e) {
                        if (callback != null) {
                            callback.onFailed(e.toString());
                        }
                        ALog.e(TAG, "uploadImage onFailed:" + e.toString());
                        appendLog("事件图片上报失败，error：" + e.toString());
                    }
                    break;
                default:
                    break;
            }
        }
    };

    private void setReportEventValid(final boolean isEnable) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                reportEventBtn.setEnabled(isEnable);
            }
        });
    }

    /**
     * 获取设备属性
     *
     * @return 属性列表
     */
    private List<Property> getAllProperties() {
        if (propertyList == null || propertyList.size() == 0) {
            propertyList = LinkKit.getInstance().getDeviceThing().getProperties();
        }
        return propertyList;
    }

    /**
     * 获取物模型服务列表
     *
     * @return 物模型服务列表
     */
    public List<Service> getServiceList() {
        if (serviceList == null || serviceList.size() == 0) {
            serviceList = LinkKit.getInstance().getDeviceThing().getServices();
        }
        return serviceList;
    }

    /**
     * 获取事件列表
     *
     * @return 事件列表
     */
    public List<Event> getAllEventList() {
        if (eventList == null || eventList.size() == 0) {
            eventList = LinkKit.getInstance().getDeviceThing().getEvents();
        }
        return eventList;
    }

    /**
     * 上报属性
     */
    private void setAllProperties() {
        Map<String, ValueWrapper> properties = new HashMap<>();
        properties.put("StreamVideoQuality", new ValueWrapper.EnumValueWrapper(2));
        properties.put("SubStreamVideoQuality", new ValueWrapper.EnumValueWrapper(1));
        properties.put("AlarmSwitch", new ValueWrapper.BooleanValueWrapper(1));
        properties.put("MotionDetectSensitivity", new ValueWrapper.EnumValueWrapper(5));
        properties.put("VoiceDetectionSensitivity", new ValueWrapper.EnumValueWrapper(5));
        properties.put("AlarmFrequencyLevel", new ValueWrapper.EnumValueWrapper(2));
        properties.put("MicSwitch", new ValueWrapper.BooleanValueWrapper(1));
        properties.put("SpeakerSwitch", new ValueWrapper.BooleanValueWrapper(1));
        properties.put("StatusLightSwitch", new ValueWrapper.BooleanValueWrapper(1));
        properties.put("ImageFlipState", new ValueWrapper.EnumValueWrapper(1));
        properties.put("StorageRecordMode", new ValueWrapper.EnumValueWrapper(2));
        properties.put("StorageStatus", new ValueWrapper.EnumValueWrapper(1));
        properties.put("StorageTotalCapacity", new ValueWrapper.DoubleValueWrapper(120.0));
        properties.put("StorageRemainCapacity", new ValueWrapper.DoubleValueWrapper(1.0));
        properties.put("DayNightMode", new ValueWrapper.EnumValueWrapper(2));
        properties.put("EncryptSwitch", new ValueWrapper.BooleanValueWrapper(1));
        ArrayList<ValueWrapper> encryptTypeList = new ArrayList<>(2);
        encryptTypeList.add(new ValueWrapper.IntValueWrapper(0));
        properties.put("EncryptTypeList", new ValueWrapper.ArrayValueWrapper(encryptTypeList));
        try {
            LinkKit.getInstance().getDeviceThing().thingPropertyPost(properties, new IPublishResourceListener() {
                @Override
                public void onSuccess(String s, Object o) {
                    ALog.d(TAG, "thingPropertyPost onSuccess() called with: s = [" + s + "], o = [" + o + "]");

                }

                @Override
                public void onError(String s, AError aError) {
                    ALog.d(TAG,
                            "thingPropertyPost onError() called with: s = [" + s + "], aError = [" + aError + "]");

                }
            });
        } catch (Exception e) {
            e.printStackTrace();

        }

    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        LinkKit.getInstance().unRegisterOnPushListener(connectNotifyListener);
    }

    /**
     * 注册所有服务监听
     */
    private void setServiceHandler() {
        for (int i = 0; serviceList != null && i < serviceList.size(); i++) {
            Service service = serviceList.get(i);
            /**
             * 监听异步服务
             */
            LinkKit.getInstance().getDeviceThing().setServiceHandler(service.getIdentifier(),
                    itResRequestHandler);
        }
        /**
         * 监听同步服务
         */
        LinkKit.getInstance().registerOnPushListener(connectNotifyListener);
    }

    private ITResRequestHandler itResRequestHandler = new ITResRequestHandler() {
        @Override
        public void onProcess(String identify, Object result, ITResResponseCallback
                itResResponseCallback) {

            ALog.d(TAG,
                    "ITResRequestHandler  onProcess() called with: identify = [" + identify + "], result = ["
                            + JSON.toJSONString(result) + "], itResResponseCallback = ["
                            + itResResponseCallback + "]");
            /**
             * 添加SDK的监听。
             */
            IPCDev.getInstance().notifyAsyncTopicReceived(identify, result, itResResponseCallback);
        }

        @Override
        public void onSuccess(Object o, OutputParams outputParams) {
            ALog.d(TAG,
                    "onSuccess() called with: o = [" + JSON.toJSONString(o) + "], outputParams = [" + JSON
                            .toJSONString(outputParams) + "]");
        }

        @Override
        public void onFail(Object o, ErrorInfo errorInfo) {
            ALog.d(TAG, "onFail() called with: o = [" + JSON.toJSONString(o) + "], errorInfo = [" + JSON
                    .toJSONString(errorInfo) + "]");
        }
    };

    private IConnectNotifyListener connectNotifyListener = new IConnectNotifyListener() {
        @Override
        public void onNotify(String connectId, String topic, AMessage aMessage) {
            ALog.d(TAG, "onNotify() called with: connectId = [" + connectId + "], topic = [" + topic + "], aMessage = ["
                    + new String((byte[]) aMessage.data) + "]");
            /**
             * 添加SDK的监听。
             */
            IPCDev.getInstance().notifySyncTopicReceived(connectId, topic, aMessage);

            // 处理同步服务调用
            if (CONNECT_ID.equals(connectId) && !TextUtils.isEmpty(topic) &&
                    topic.contains("rrpc")) {
                int code = 200;
                String data = "{}";

                JSONObject json = JSON.parseObject(new String((byte[]) aMessage.data));
                if (json != null) {
                    String method = json.getString("method");
                    JSONObject params = json.getJSONObject("params");
                    switch (method) {
                        // 查询卡录像列表请求
                        case "thing.service.QueryRecordTimeList":
                        case "thing.service.QueryRecordList":
                            int beginTime = params.getIntValue("BeginTime");
                            int endTime = params.getIntValue("EndTime");
                            int querySize = params.getIntValue("QuerySize");
                            int type = params.getIntValue("Type");
                            appendLog("收到查询卡录像列表的请求: beginTime=" + beginTime +
                                    "\tendTime=" + endTime + "\tquerySize=" + querySize + "\ttype=" + type);

                            JSONArray resultArray = new JSONArray();
                            JSONObject item1 = new JSONObject();
                            item1.put("FileName", new String(Base64.encode("file1".getBytes(), Base64.NO_WRAP)));
                            item1.put("BeginTime", System.currentTimeMillis() / 1000 - 200);
                            item1.put("EndTime", System.currentTimeMillis() / 1000 - 100);
                            item1.put("Size", 1024000);
                            item1.put("Type", 0);
                            resultArray.add(item1);

                            JSONObject item2 = new JSONObject();
                            item2.put("FileName", new String(Base64.encode("file2".getBytes(), Base64.NO_WRAP)));
                            item2.put("BeginTime", System.currentTimeMillis() / 1000 - 100);
                            item2.put("EndTime", System.currentTimeMillis() / 1000);
                            item2.put("Size", 1024000);
                            item2.put("Type", 0);
                            resultArray.add(item2);

                            JSONObject result = new JSONObject();
                            result.put("RecordList", resultArray);

                            code = 200;
                            data = result.toJSONString();
                            break;
                        default:
                            break;
                    }
                }

                MqttPublishRequest request = new MqttPublishRequest();
                request.isRPC = false;
                request.topic = topic.replace("request", "response");
                String resId = topic.substring(topic.indexOf("rrpc/request/") + 13);
                request.msgId = resId;
                request.payloadObj = "{\"id\":\"" + resId + "\", \"code\":" + code + ",\"data\":" + data + "}";
                LinkKit.getInstance().publish(request, new IConnectSendListener() {
                    @Override
                    public void onResponse(ARequest aRequest, AResponse aResponse) {
                        appendLog("上报成功");
                    }

                    @Override
                    public void onFailure(ARequest aRequest, AError aError) {
                        appendLog("上报失败:" + aError.toString());
                    }
                });
            } else if (CONNECT_ID.equals(connectId) && !TextUtils.isEmpty(topic) &&
                    topic.contains("thing/service/property/set")) {
                // 处理属性设置
                JSONObject json = JSON.parseObject(new String((byte[]) aMessage.data));
                if (json != null) {
                    JSONObject params = json.getJSONObject("params");
                    if (params != null && params.containsKey("StreamVideoQuality")) {
                        int streamVideoQuality = params.getInteger("StreamVideoQuality");
                        appendLog("切换到清晰度" + streamVideoQuality);
                        switchStream(currentLiveStreamId, streamVideoQuality);
                    }
                }
            }
        }

        @Override
        public boolean shouldHandle(String connectId, String topic) {
            return true;
        }

        @Override
        public void onConnectStateChange(String connectId, ConnectState connectState) {

        }
    };

    @Override
    protected void onStop() {
        super.onStop();
        // 页面退出停止推流
        stopAllStreaming();
    }

    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private void appendLog(final String msg) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                logTv.append(format.format(new Date()) + "\t" + msg);
                logTv.append("\n");
                logScroll.fullScroll(View.FOCUS_DOWN);
            }
        });
    }



    /* Stream related below */

    public static final String H264_720P_FILE_NAME = "720p.h264";
    public static final int H264_720P_FILE_LENGTH = 40275588;
    public static final String H264_480P_FILE_NAME = "480p.h264";
    public static final int H264_480P_FILE_LENGTH = 48463205;
    public static final int H264_DURATION_IN_S = 180;
    public static final String G711A_FILE_NAME = "8k_1c.g711a";
    public static final int G711A_FILE_LENGTH = 1440566;
    public static final String AAC_FILE_NAME = "8k_2c.aac";
    public static final int AAC_FILE_LENGTH = 40127;

    private HashMap<Integer, SendStreamingTask> sendTaskMap = new HashMap<>();
    private int currentLiveStreamId;
    private byte[] h264HighQualityData;
    private byte[] h264LowQualityData;
    private byte[] g711aData;
    private byte[] aacData;

    @Override
    public void onError(int streamId, StreamError error) {
        error.printStackTrace();
        stopStreaming(streamId);
        appendLog("流ID:" + streamId + " 发生错误：" + error.getCode());
    }

    /**
     * 收到开始推直播流请求
     *
     * @param streamId   流ID
     * @param streamType 码流类型: 0为主码流, 1为辅码流
     * @param preTimeInS 预先录制时间，单位S
     */
    @Override
    public void onStartPushLiveStreaming(int streamId, int streamType, int preTimeInS) {
        currentLiveStreamId = streamId;
        appendLog("开始直播推流 " + streamId);
        try {

            // 构造视频参数
            VideoStreamParams videoStreamParams = new VideoStreamParams();
            videoStreamParams.setDurationInS(0);
            videoStreamParams.setVideoFormat(VideoStreamParams.VIDEO_FORMAT_H264);

            // 构造音频参数
            AudioStreamParams audioStreamParams = new AudioStreamParams();
            audioStreamParams.setAudioSampleRate(AudioStreamParams.AUDIO_SAMPLE_RATE_8000);
            audioStreamParams.setAudioChannel(AudioStreamParams.AUDIO_CHANNEL_MONO);
            audioStreamParams.setAudioFormat(AudioStreamParams.AUDIO_FORMAT_G711A);
            audioStreamParams.setAudioEncoding(AudioStreamParams.AUDIO_ENCODING_16BIT);

            // 设置推流参数
            IPCDev.getInstance().getIpcStreamManager().setStreamParams(streamId, videoStreamParams, audioStreamParams);

            // 开线程模拟音视频数据的生产和发送
            SendStreamingTask sendLiveStreamingTask = new SendStreamingTask(
                    new PushLiveStreamingDataRunnable(streamId, 2));
            sendLiveStreamingTask.start();
            sendTaskMap.put(streamId, sendLiveStreamingTask);
        } catch (NoSuchStreamException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onStartPushVodStreaming(int streamId, String fileName) {
        appendLog("开始推点播流 " + streamId + " 文件名:" + new String(Base64.decode(fileName, Base64.NO_WRAP)));

        try {
            // 构造视频参数
            VideoStreamParams videoStreamParams = new VideoStreamParams();
            videoStreamParams.setDurationInS(H264_DURATION_IN_S);
            videoStreamParams.setVideoFormat(VideoStreamParams.VIDEO_FORMAT_H264);

            // 构造音频参数
            AudioStreamParams audioStreamParams = new AudioStreamParams();
            audioStreamParams.setAudioChannel(AudioStreamParams.AUDIO_CHANNEL_STEREO);
            audioStreamParams.setAudioFormat(AudioStreamParams.AUDIO_FORMAT_AAC);
            audioStreamParams.setAudioEncoding(AudioStreamParams.AUDIO_ENCODING_16BIT);
            audioStreamParams.setAudioSampleRate(AudioStreamParams.AUDIO_SAMPLE_RATE_8000);

            // 设置推流参数
            IPCDev.getInstance().getIpcStreamManager().setStreamParams(streamId, videoStreamParams, audioStreamParams);

            // 开线程模拟音视频数据的生产和发送
            SendStreamingTask sendVodStreamingTask = new SendStreamingTask(new PushVodStreamingDataRunnable(streamId));
            sendVodStreamingTask.start();
            sendTaskMap.put(streamId, sendVodStreamingTask);
        } catch (NoSuchStreamException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onStartPushVodStreaming(int streamId, int beginTimeUtc, int endTimeUtc) {
        appendLog("开始推点播流 " + streamId + " beginTimeUtc: "+beginTimeUtc + " endTimeUtc:"+endTimeUtc);

        //FIXME 推流逻辑需要添加:
        // 1. beginTimeUtc和endTimeUtc一般是一天的开始和结束时间
        // 2. 当收到onStartPushVodStreaming回调后，应从beginTimeUtc开始向后最近的I帧开始推流，时间戳应使用对应帧的UTC时间
        // 3. 若beginTimeUtc到endTimeUtc范围内没有录像或范围内推流已经完成了，则应调用 IPCDev.getInstance().getIpcStreamManager().notifyVodComplete(streamId) 通知推流完成
        // 4. 只要是beginTimeUtc到endTimeUtc范围内有数据, 即使跨文件，推流应该持续不断

        try {
            // FIXME 当前没有文件推流，所以直接通知推流完成了
            IPCDev.getInstance().getIpcStreamManager().notifyVodComplete(streamId);
        } catch (NoSuchStreamException e) {
            e.printStackTrace();
        }
    }

    /**
     * 收到停止推流请求
     *
     * @param streamId 流ID
     */
    @Override
    public void onStopPushStreaming(int streamId) {
        appendLog("停止推流 " + streamId);
        stopStreaming(streamId);
    }

    @Override
    public void onPausePushVodStreaming(int streamId) {
        appendLog("暂停推流 " + streamId);
        sendTaskMap.get(streamId).getSendRunnable().pause();
    }

    @Override
    public void onResumePushVodStreaming(int streamId) {
        appendLog("恢复推流 " + streamId);
        sendTaskMap.get(streamId).getSendRunnable().resume();
    }

    @Override
    public void onSeekTo(int streamId, long timeStampInS) {
        appendLog("seek流 " + streamId + " timeStampInS:" + timeStampInS);
        if(sendTaskMap.get(streamId) != null ) {
            sendTaskMap.get(streamId).getSendRunnable().seek((int) timeStampInS);
        }
    }

    /**
     * 收到强制I帧请求
     *
     * @param streamId 流ID
     */
    @Override
    public void onForceIFrame(int streamId) {
        appendLog("强制I帧 " + streamId);
    }

    private void stopStreaming(int streamId) {
        if (sendTaskMap.get(streamId) != null) {
            sendTaskMap.get(streamId).getSendRunnable().finish();
            sendTaskMap.get(streamId).join(3000);
            sendTaskMap.remove(streamId);
        }
        try {
            IPCDev.getInstance().getIpcStreamManager().stopStreaming(streamId);
        } catch (NoSuchStreamException e) {
            e.printStackTrace();
        }
    }

    private void stopAllStreaming() {
        for (int i : sendTaskMap.keySet()) {
            stopStreaming(i);
        }
    }

    /**
     * 直播数据推流任务
     * 示例: 视频为H264、音频为G711a
     */
    class PushLiveStreamingDataRunnable implements SendStreamingTask.SendRunnable {

        private boolean isFinish;
        private int streamId;
        private int streamQuality;

        ByteBuffer videoByteBuffer = ByteBuffer.allocateDirect(1024*1024);
        ByteBuffer audioByteBuffer = ByteBuffer.allocateDirect(320);

        PushLiveStreamingDataRunnable(int streamId, int streamQuality) {
            this.streamId = streamId;
            this.streamQuality = streamQuality;
        }

        @Override
        public int getStreamId() {
            return streamId;
        }

        @Override
        public void resume() {

        }

        @Override
        public void pause() {

        }

        @Override
        public void seek(int newPosition) {

        }

        @Override
        public void finish() {
            isFinish = true;
        }

        @Override
        public void run() {
            byte[] livingH264Data = (streamQuality == 2) ? h264HighQualityData : h264LowQualityData;
            long ts = System.currentTimeMillis();
            long timestamp = 0L;
            long deltaTimeInMs = 40;
            int audioIndex = 44;
            ByteBuffer sps = ByteBuffer.allocate(1024);
            ByteBuffer pps = ByteBuffer.allocate(1024);
            int count = 0;
            for (int i = 0; i < livingH264Data.length && !isFinish; ) {
                int index = MockUtils.findNalHeader(livingH264Data, i, livingH264Data.length);
                int nextIndex = MockUtils.findNalHeader(livingH264Data, i + 3, livingH264Data.length);
                if ((i == 0 && index == 0) || index > 0) {
                    if (nextIndex == -1) {
                        nextIndex = livingH264Data.length;
                    }
                    ALog.d(TAG, "isIframe:" + MockUtils.isIFrame(livingH264Data, index) + "\t isSps:" + MockUtils
                            .isSps(livingH264Data, index) + "\t isPPS:" + MockUtils.isPps(livingH264Data, index));
                    ALog.d(TAG, "i:" + i + " len:" + livingH264Data.length + " index:" + index + " nextIndex:" + nextIndex);
                    // 更新SPS
                    if (MockUtils.isSps(livingH264Data, index)) {
                        sps.clear();
                        sps.put(livingH264Data, index, nextIndex-index);
                        sps.flip();
                        i = nextIndex;
                        continue;
                    }
                    // 更新PPS
                    if (MockUtils.isPps(livingH264Data, index)) {
                        pps.clear();
                        pps.put(livingH264Data, index, nextIndex-index);
                        pps.flip();
                        i = nextIndex;
                        continue;
                    }

                    if ((audioIndex + audioByteBuffer.capacity()) < g711aData.length) {
                        long ms1 = System.currentTimeMillis();
                        try {
                            // 发送音频帧
                            audioByteBuffer.clear();
                            audioByteBuffer.put(g711aData, audioIndex, audioByteBuffer.capacity());
                            audioByteBuffer.flip();
                            IPCDev.getInstance().getIpcStreamManager().sendAudioData(streamId, audioByteBuffer,
                                    audioByteBuffer.capacity(), timestamp);
                        } catch (NoSuchStreamException e) {
                            e.printStackTrace();
                            break;
                        }
                        ALog.d(TAG, "sendAudio " + (System.currentTimeMillis() - ms1) + " ms");
                        audioIndex += audioByteBuffer.capacity();
                    }

                    if (MockUtils.isIFrame(livingH264Data, index)) {
                        long ms2 = System.currentTimeMillis();
                        // 构造包含SPS和PPS的大I帧
                        videoByteBuffer.clear();
                        videoByteBuffer.put(sps);
                        videoByteBuffer.put(pps);
                        videoByteBuffer.put(livingH264Data, index, nextIndex - index);
                        videoByteBuffer.flip();
                        try {
                            // 发送I帧(需包括SPS,PPS)
                            IPCDev.getInstance().getIpcStreamManager().sendVideoData(streamId, videoByteBuffer,  videoByteBuffer.limit(),
                                    true, timestamp);
                        } catch (NoSuchStreamException e) {
                            e.printStackTrace();
                            break;
                        }
                        ALog.d("sendIFrame", (System.currentTimeMillis() - ms2) + " ms");
                    } else if (MockUtils.isPFrame(livingH264Data, index)) {
                        long ms3 = System.currentTimeMillis();
                        try {
                            // 发送P帧
                            videoByteBuffer.clear();
                            videoByteBuffer.put(livingH264Data, index, nextIndex - index);
                            videoByteBuffer.flip();
                            IPCDev.getInstance().getIpcStreamManager().sendVideoData(streamId, videoByteBuffer,
                                    videoByteBuffer.limit(), false, timestamp);
                        } catch (NoSuchStreamException e) {
                            e.printStackTrace();
                            break;
                        }
                        ALog.d("sendFrame", (System.currentTimeMillis() - ms3) + " ms");
                    }
                    timestamp += deltaTimeInMs;
                    // 适当sleep来模拟实时的数据生产
                    long totalSendTime = (System.currentTimeMillis() - ts);
                    long sleeptime = deltaTimeInMs * ++count - totalSendTime;
                    if (sleeptime < 0) {
                        ts = Math.abs(sleeptime) + ts;
                    } else {
                        try {
                            Thread.sleep(sleeptime);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                    i = nextIndex;
                    // 无限循环发送数据
                    if (i >= livingH264Data.length) {
                        ALog.e(TAG, "video loop end count=" + count);
                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                        i = 0;
                        audioIndex = 44;
                    }
                } else {
                    ALog.d(TAG, " break:" + i);
                    break;
                }
            }

        }
    }

    /**
     * 点播数据推流任务
     * 示例: 视频为H264、音频为AAC_LC
     */
    class PushVodStreamingDataRunnable implements SendStreamingTask.SendRunnable {

        private Object pauseLock;
        private boolean paused;
        private boolean isFinish;

        private AtomicBoolean seeked = new AtomicBoolean(false);
        private int newPosition = 0;

        private int streamId = 0;

        PushVodStreamingDataRunnable(int streamId) {
            this.pauseLock = new Object();
            this.streamId = streamId;
        }

        @Override
        public int getStreamId() {
            return streamId;
        }

        @Override
        public void resume() {
            synchronized (pauseLock) {
                paused = false;
                pauseLock.notifyAll();
            }
        }

        @Override
        public void pause() {
            synchronized (pauseLock) {
                paused = true;
            }
        }

        @Override
        public void seek(int newPosition) {
            this.newPosition = newPosition;
            seeked.set(true);
        }

        @Override
        public void finish() {
            resume();
            isFinish = true;
        }

        @Override
        public void run() {
            long ts = System.currentTimeMillis();
            long timestamp = 0L;
            long deltaTimeInMs = 40;
            int audioIndex = 0;
            byte[] sps = null;
            byte[] pps = null;
            int count = 0;
            for (int i = 0; i < h264HighQualityData.length && !isFinish; ) {
                synchronized (pauseLock) {
                    while (paused) {
                        try {
                            pauseLock.wait();
                        } catch (InterruptedException e) {
                        }
                    }
                }
                if (seeked.compareAndSet(true, false)) {
                    timestamp = newPosition * 1000;
                }
                int index = MockUtils.findNalHeader(h264HighQualityData, i, h264HighQualityData.length);
                int nextIndex = MockUtils.findNalHeader(h264HighQualityData, i + 3, h264HighQualityData.length);
                if ((i == 0 && index == 0) || index > 0) {
                    if (nextIndex == -1) {
                        nextIndex = h264HighQualityData.length;
                    }
                    ALog.d(TAG, "isIframe:" + MockUtils.isIFrame(h264HighQualityData, index) + "\t isSps:" + MockUtils
                            .isSps(h264HighQualityData, index) + "\t isPPS:" + MockUtils.isPps(h264HighQualityData, index));
                    ALog.d(TAG, "i:" + i + " len:" + h264HighQualityData.length + " index:" + index + " nextIndex:" + nextIndex);
                    // 更新SPS
                    if (MockUtils.isSps(h264HighQualityData, index)) {
                        sps = new byte[nextIndex - index];
                        System.arraycopy(h264HighQualityData, index, sps, 0, nextIndex - index);
                        i = nextIndex;
                        continue;
                    }
                    // 更新PPS
                    if (MockUtils.isPps(h264HighQualityData, index)) {
                        pps = new byte[nextIndex - index];
                        System.arraycopy(h264HighQualityData, index, pps, 0, nextIndex - index);
                        i = nextIndex;
                        continue;
                    }

                    int audioHeaderIndex = MockUtils.findADTSHeader(aacData, audioIndex, AAC_FILE_LENGTH);
                    int audioNextHeaderIndex = MockUtils.findADTSHeader(aacData, audioIndex + 1, AAC_FILE_LENGTH);
                    if (audioNextHeaderIndex == -1) {
                        audioIndex = 0;
                    } else {
                        if ((audioIndex == 0 && audioHeaderIndex == 0) || audioHeaderIndex > 0) {
                            long ms1 = System.currentTimeMillis();
                            try {
                                //发送音频帧, AAC raw data, 不含ADTS头
                                IPCDev.getInstance().getIpcStreamManager().sendAudioData(streamId, aacData, audioHeaderIndex + 7,
                                        audioNextHeaderIndex - audioHeaderIndex - 7, timestamp);
                            } catch (NoSuchStreamException e) {
                                e.printStackTrace();
                                break;
                            }
                            audioIndex = audioNextHeaderIndex;
                            ALog.d("sendAudio", (System.currentTimeMillis() - ms1) + " ms");
                        }
                    }

                    if (MockUtils.isIFrame(h264HighQualityData, index)) {
                        long ms2 = System.currentTimeMillis();
                        // 构造包含SPS和PPS的大I帧
                        byte[] iFrame = new byte[nextIndex - index + sps.length + pps.length];
                        System.arraycopy(sps, 0, iFrame, 0, sps.length);
                        System.arraycopy(pps, 0, iFrame, sps.length, pps.length);
                        System.arraycopy(h264HighQualityData, index, iFrame, sps.length + pps.length, nextIndex - index);
                        try {
                            // 发送I帧(需包括SPS,PPS)
                            IPCDev.getInstance().getIpcStreamManager().sendVideoData(streamId, iFrame, 0, iFrame.length,
                                    true, timestamp);
                        } catch (NoSuchStreamException e) {
                            e.printStackTrace();
                            break;
                        }
                        ALog.d("sendIFrame", (System.currentTimeMillis() - ms2) + " ms");
                    } else if (MockUtils.isPFrame(h264HighQualityData, index)) {
                        long ms3 = System.currentTimeMillis();
                        try {
                            // 发送P帧
                            IPCDev.getInstance().getIpcStreamManager().sendVideoData(streamId, h264HighQualityData, index,
                                    nextIndex - index, false, timestamp);
                        } catch (NoSuchStreamException e) {
                            e.printStackTrace();
                            break;
                        }
                        ALog.d("sendFrame", (System.currentTimeMillis() - ms3) + " ms");
                    }
                    timestamp += deltaTimeInMs;
                    // 适当sleep来模拟实时的数据生产
                    long totalSendTime = (System.currentTimeMillis() - ts);
                    long sleeptime = deltaTimeInMs * ++count - totalSendTime;
                    ALog.d(TAG, "sleep sync:" + sleeptime + " ms ");
                    try {
                        Thread.sleep(sleeptime > 0 ? sleeptime : 0);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    i = nextIndex;
                } else {
                    ALog.d(TAG, " break:" + i);
                    break;
                }
            }

        }
    }

    private void switchStream(int streamId, int streamQuality) {
        SendStreamingTask sendStreamingTask = sendTaskMap.get(streamId);
        sendStreamingTask.getSendRunnable().finish();
        sendStreamingTask.join(1000);
        SendStreamingTask sendLiveStreamingTask = new SendStreamingTask(
                new PushLiveStreamingDataRunnable(streamId, streamQuality));
        sendLiveStreamingTask.start();
        sendTaskMap.put(streamId, sendLiveStreamingTask);
    }


    /* Live Intercom related below */

    private OnLiveIntercomListener onLiveIntercomListener = new OnLiveIntercomListener(){

        @Override
        public AudioParams onStartVoiceIntercom() {
            appendLog("收到开始语音对讲指令");
            return new AudioParams(48000, 1, AudioFormat.ENCODING_PCM_16BIT, AudioParams.TYPE_G711A);
        }

        @Override
        public void onStopVoiceIntercom() {
            appendLog("收到停止语音对讲指令");
        }

        @Override
        public void onAudioParamsChange(AudioParams audioParams) {
            appendLog("收到客户端的音频参数: " + audioParams.toString());
        }

        @Override
        public void onAudioBufferReceive(byte[] buffer, int size) {
            // ignore
        }
    };

    private OnLiveIntercomErrorListener onLiveIntercomErrorListener = new OnLiveIntercomErrorListener() {
        @Override
        public void onError(LiveIntercomError error) {
            appendLog("语音对讲错误:" + error.getCode() + " msg:" + error.getMessage());
        }
    };

    public static void verifyPermissions(String[] permissions, Activity activity) {
        List<String> permissionsNeed = new ArrayList<>();
        try {
            for (String permission : permissions) {
                int granted = ActivityCompat.checkSelfPermission(activity,
                        permission);
                if (granted != PackageManager.PERMISSION_GRANTED) {
                    permissionsNeed.add(permission);
                }
            }

            if (!permissionsNeed.isEmpty()) {
                ActivityCompat.requestPermissions(activity, permissionsNeed.toArray(new String[permissionsNeed.size()]),
                        1);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
