package com.aliyun.iotx.linkvisual.ipc.utils;

import android.text.TextUtils;

import com.aliyun.alink.linksdk.tools.ALog;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

public class EnvUtils {

    // FIXME 可以设置默认的三元组
    public static String DEFAULT_PRODUCT_KEY = "";
    public static String DEFAULT_DEVICE_NAME = "";
    public static String DEFAULT_DEVICE_SECRET = "";

    public static final String DEBUG_LV_PK = "debug.lv.pk";
    public static final String DEBUG_LV_DN = "debug.lv.dn";
    public static final String DEBUG_LV_DS = "debug.lv.ds";

    private static String GETPROP_EXECUTABLE_PATH = "/system/bin/getprop";
    private static String TAG = "linksdk_lv_EnvUtils";

    public static String getProductKey(){
        String pk = readProp(DEBUG_LV_PK);
        ALog.d(TAG, DEBUG_LV_PK +": "+ pk);
        if(TextUtils.isEmpty(pk)){
            return DEFAULT_PRODUCT_KEY;
        }else{
            return pk;
        }
    }

    public static String getDeviceName(){
        String dn = readProp(DEBUG_LV_DN);
        ALog.d(TAG, DEBUG_LV_DN +": "+ dn);
        if(TextUtils.isEmpty(dn)){
            return DEFAULT_DEVICE_NAME;
        }else{
            return dn;
        }
    }

    public static String getDeviceSecret(){
        String ds = readProp(DEBUG_LV_DS);
        ALog.d(TAG, DEBUG_LV_DS +": "+ ds);
        if(TextUtils.isEmpty(ds)){
            return DEFAULT_DEVICE_SECRET;
        }else{
            return ds;
        }
    }

    public static String readProp(String propName) {
        Process process = null;
        BufferedReader bufferedReader = null;

        try {
            process = new ProcessBuilder().command(GETPROP_EXECUTABLE_PATH, propName).redirectErrorStream(true).start();
            bufferedReader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line = bufferedReader.readLine();
            return line;
        } catch (Exception e) {
            ALog.e(TAG,"Failed to read System Property " + propName,e);
            return null;
        } finally{
            if (bufferedReader != null){
                try {
                    bufferedReader.close();
                } catch (IOException e) {}
            }
            if (process != null){
                process.destroy();
            }
        }
    }

    public static String help() {
        return "\n===============================================\n" +
                "当前使用三元组:\n" +
                "pk:" + getProductKey() + "\t dn:" + getDeviceName() + "\n" +
                "===============================================\n" +
                "若要更改三元组配置,请通过adb执行命令: \n" +
                "adb shell setprop " + DEBUG_LV_PK + " xxxx\n" +
                "adb shell setprop " + DEBUG_LV_DN + " xxxx\n" +
                "adb shell setprop " + DEBUG_LV_DS + " xxxx\n" +
                "以上设置需重启app后生效\n" +
                "===============================================\n\n";
    }
}
