package com.aliyun.iotx.linkvisual.ipc.utils;

import android.content.Context;

import java.io.IOException;
import java.io.InputStream;

public class MockUtils {

    public static byte[] readFileAsByteArrays(Context context, String fileName, int length) {
        byte[] readBuffer = null;
        try {
            readBuffer = new byte[length];
            InputStream inputStream = context.getAssets().open(fileName);
            inputStream.read(readBuffer);
            inputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return readBuffer;
    }

    public static int findNalHeader(byte[] data, int startIndex, int max) {
        int i = -2;
        for (i = startIndex; i < max; i++) {
            if (isNalHeader(data, i)) {
                break;
            }
        }
        if (i == max) {
            i = -1;
        }
        return i;
    }

    public static boolean isNalHeader(byte[] data, int offset) {
        boolean result = false;
        if (data[offset] == (byte) 0x00 && data[offset + 1] == (byte) 0x00 && ((data[offset + 2] == (byte) 0x00 && data[offset + 3] == (byte) 0x01) || data[offset + 2] == (byte) 0x01)) {
            result = true;
        }
        return result;
    }

    public static boolean isSps(byte[] data, int offset) {
        boolean result = false;
        if (isNalHeader(data, offset) && ((data[offset + 4] == (byte) 0x67) || (data[offset + 3] == (byte) 0x67) || (data[offset + 4] == (byte) 0x42) || (data[offset + 3] == (byte) 0x42))) {
            result = true;
        }
        return result;
    }

    public static boolean isPps(byte[] data, int offset) {
        boolean result = false;
        if (isNalHeader(data, offset) && ((data[offset + 4] == (byte) 0x68) || (data[offset + 3] == (byte) 0x68) || (data[offset + 4] == (byte) 0x44) || (data[offset + 3] == (byte) 0x44))) {
            result = true;
        }
        return result;
    }

    public static boolean isVps(byte[] data, int offset) {
        boolean result = false;
        if (isNalHeader(data, offset) && ((data[offset + 4] == (byte) 0x40) || (data[offset + 3] == (byte) 0x40))) {
            result = true;
        }
        return result;
    }

    public static boolean isIFrame(byte[] data, int offset) {
        boolean result = false;
        if (isNalHeader(data, offset) && ((data[offset + 4] == (byte) 0x65) || (data[offset + 3] == (byte) 0x65) || (data[offset + 4] == (byte) 0x26) || (data[offset + 3] == (byte) 0x26))) {
            result = true;
        }
        return result;
    }


    public static boolean isPFrame(byte[] data, int offset) {
        boolean result = false;
        if (isNalHeader(data, offset) &&
                ((data[offset + 4] == (byte) 0x41) || (data[offset + 3] == (byte) 0x41) ||
                        (data[offset + 3] == (byte) 0x61) || (data[offset + 4] == (byte) 0x61) ||
                        (data[offset + 3] == (byte) 0x21) || (data[offset + 4] == (byte) 0x21))) {
            result = true;
        }
        return result;
    }

    public static int findADTSHeader(byte[] data, int startIndex, int max) {
        int i = -2;
        for (i = startIndex; i < max; i++) {
            if (isADTSHeader(data, i)) {
                break;
            }
        }
        if (i == max) {
            i = -1;
        }
        return i;

    }

    private static boolean isADTSHeader(byte[] data, int offset) {
        boolean result = false;
        if (data[offset] == (byte) 0xFF && (data[offset + 1] == (byte) 0xF9 || data[offset + 1] == (byte) 0xF1)
                && data[offset + 3] == (byte) 0x80) {
            result = true;
        }
        return result;
    }
}
