package com.aliyun.iotx.linkvisual.ipc.utils;


import java.io.File;
import java.io.IOException;
import java.util.concurrent.TimeUnit;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

public class UploadPicUtil {

    /**
     * 上传图片.
     *
     * @param url       上传url
     * @param imagePath 图片路径
     * @param imageType 图片类型，满足MIME类型定义
     * @param listener  上传经过回调
     * @throws IOException the io exception
     */
    public static void uploadImage(String url, String imagePath, String imageType,
                                   final UploadResultCallback listener) throws
        IOException {
        OkHttpClient okHttpClient = new OkHttpClient.Builder()
            .connectTimeout(600, TimeUnit.SECONDS)
            .readTimeout(200, TimeUnit.SECONDS)
            .writeTimeout(600, TimeUnit.SECONDS)
            .build();

        File file = new File(imagePath);
        RequestBody image = RequestBody.create(MediaType.parse(imageType), file);
        Request request = new Request.Builder()
            .url(url)
            .put(image)
            .build();
        okHttpClient.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                if (listener != null) {
                    listener.onFailed(e.toString());
                }

                call.cancel();
            }

            @Override
            public void onResponse(Call call, Response response) {
                if (listener != null) {
                    listener.onSucceed();
                }

                call.cancel();
            }
        });

    }

    /**
     * 上传监听接口
     *
     */
    public interface UploadResultCallback {
        /**
         * 上传成功
         *
         */
        void onSucceed();

        /**
         * 上传失败
         *
         * @param error 错误信息
         */
        void onFailed(String error);
    }
}
